/*
 * Copyright (C) 2023 Splash authors
 *
 * This file is part of Splash.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Splash is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Splash.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @gui_gfx_impl.h
 * Implementation of class for graphics calls used in the Gui class, for OpenGL
 */

#ifndef SPLASH_OPENGL_GUI_GFX_IMPL_H
#define SPLASH_OPENGL_GUI_GFX_IMPL_H

#include <string>
#include <string_view>

#include "./core/constants.h"
#include "./graphics/api/gui_gfx_impl.h"

struct ImDrawData;

namespace Splash::gfx::opengl
{

class GuiGfxImpl : public gfx::GuiGfxImpl
{
  public:
    /**
     * Constructor
     */
    GuiGfxImpl() = default;

    /**
     * Destructor
     */
    ~GuiGfxImpl() override;

    /**
     * Initialize the shaders and vertex buffer
     */
    void initRendering() override final;

    /**
     * Initialize fonts texture
     * \param width Font texture width
     * \param height Font texture height
     * \param pixels Font texture data
     * \return Return the ID for the texture font
     */
    ImTextureID initFontTexture(uint32_t width, uint32_t height, uint8_t* pixels) override final;

    /**
     * Setup the rendering viewport
     * \param width Viewport width
     * \param height Viewport height
     */
    void setupViewport(uint32_t width, uint32_t height) override final;

    /**
     * Render the GUI
     * \param width Display width
     * \param height Display height
     * \param drawData Data to draw, as generated by ImGui
     */
    void drawGui(float width, float height, ImDrawData* drawData) override final;

  private:
    GLuint _imFontTextureId;
    GLuint _imGuiShaderHandle, _imGuiVertHandle, _imGuiFragHandle;
    GLint _imGuiTextureLocation;
    GLint _imGuiProjMatrixLocation;
    GLint _imGuiPositionLocation;
    GLint _imGuiUVLocation;
    GLint _imGuiColorLocation;
    GLuint _imGuiVboHandle, _imGuiElementsHandle, _imGuiVaoHandle;
};

} // namespace Splash::gfx::opengl

#endif
